@@ -23,79 +23,55 @@
  */
 #include "settings.h"
 #include <libxfce4ui/libxfce4ui.h>
+#include <math.h>
 
-static void
-default_command (const gchar **command, gboolean *in_terminal, gboolean *startup_notification)
+static const GdkRGBA default_colors[NUM_COLORS] =
 {
-    gchar *s = g_find_program_in_path ("xfce4-taskmanager");
-    if (s != NULL)
-    {
-        g_free (s);
-        *command = "xfce4-taskmanager";
-        *in_terminal = FALSE;
-        *startup_notification = TRUE;
-    }
-    else
-    {
-        *command = "top";
-        *in_terminal = TRUE;
-        *startup_notification = FALSE;
-    }
-}
+    [BG_COLOR]         = {1.0, 1.0, 1.0, 0.0},
+    [FG_COLOR1]        = {0.0, 1.0, 0.0, 1.0},
+    [FG_COLOR2]        = {1.0, 0.0, 0.0, 1.0},
+    [FG_COLOR3]        = {0.0, 0.0, 1.0, 1.0},
+    [BARS_COLOR]       = {1.0, 0.73048, 0.0, 1.0},
+    [SMT_ISSUES_COLOR] = {0.9, 0, 0, 1},
+};
+
+static const gchar *const color_keys[NUM_COLORS] =
+{
+    [BG_COLOR]         = "Background",
+    [FG_COLOR1]        = "Foreground1",
+    [FG_COLOR2]        = "Foreground2",
+    [FG_COLOR3]        = "Foreground3",
+    [BARS_COLOR]       = "BarsColor",
+    [SMT_ISSUES_COLOR] = "SmtIssuesColor",
+};
 
 void
 read_settings (XfcePanelPlugin *plugin, CPUGraph *base)
 {
-    const char *value;
     char *file;
     XfceRc *rc;
 
     guint rate = 0;
     gboolean nonlinear = FALSE;
-    guint mode = 0;
+    CPUGraphMode mode = MODE_NORMAL;
     guint color_mode = 0;
-    gboolean frame = TRUE;
+    gboolean frame = FALSE;
     gboolean border = TRUE;
     gboolean bars = TRUE;
+    gboolean highlight_smt = HIGHLIGHT_SMT_BY_DEFAULT;
     guint tracked_core = 0;
 
-    GdkRGBA foreground1;
-    GdkRGBA foreground2;
-    GdkRGBA foreground3;
-    GdkRGBA background;
-    GdkRGBA barscolor;
+    GdkRGBA colors[NUM_COLORS];
     guint size;
-    const gchar  *associated_command;
-    gboolean in_terminal;
-    gboolean startup_notification;
+    gchar *command = NULL;
+    gboolean in_terminal = TRUE;
+    gboolean startup_notification = FALSE;
+    guint i, load_threshold = 0;
 
-    foreground1.red = 0.0;
-    foreground1.green = 1.0;
-    foreground1.blue = 0.0;
-    foreground1.alpha = 1.0;
-
-    foreground2.red = 1.0;
-    foreground2.green = 0.0;
-    foreground2.blue = 0.0;
-    foreground2.alpha = 1.0;
-
-    foreground3.red = 0.0;
-    foreground3.green = 0.0;
-    foreground3.blue = 1.0;
-    foreground3.alpha = 1.0;
-
-    background.red = 1.0;
-    background.green = 1.0;
-    background.blue = 1.0;
-    background.alpha = 1.0;
-
-    barscolor.red = 1.0;
-    barscolor.green = 0.73048;
-    barscolor.blue = 0.0;
-    barscolor.alpha = 1.0;
+    for (i = 0; i < NUM_COLORS; i++)
+        colors[i] = default_colors[i];
 
     size = xfce_panel_plugin_get_size (plugin);
-    default_command (&associated_command, &in_terminal, &startup_notification);
 
     if ((file = xfce_panel_plugin_lookup_rc_file (plugin)) != NULL)
     {
@@ -104,30 +80,46 @@ read_settings (XfcePanelPlugin *plugin, CPUGraph *base)
 
         if (rc)
         {
+            const gchar *value;
+
             rate =  xfce_rc_read_int_entry (rc, "UpdateInterval", rate);
             nonlinear = xfce_rc_read_int_entry (rc, "TimeScale", nonlinear);
             size = xfce_rc_read_int_entry (rc, "Size", size);
             mode = xfce_rc_read_int_entry (rc, "Mode", mode);
             color_mode = xfce_rc_read_int_entry (rc, "ColorMode", color_mode);
             frame = xfce_rc_read_int_entry (rc, "Frame", frame);
-            associated_command = g_strdup (xfce_rc_read_entry (rc, "Command", associated_command));
             in_terminal = xfce_rc_read_int_entry (rc, "InTerminal", in_terminal);
             startup_notification = xfce_rc_read_int_entry (rc, "StartupNotification", startup_notification);
             border = xfce_rc_read_int_entry (rc, "Border", border);
             bars = xfce_rc_read_int_entry (rc, "Bars", bars);
+            highlight_smt = xfce_rc_read_int_entry (rc, "SmtIssues", highlight_smt);
             tracked_core = xfce_rc_read_int_entry (rc, "TrackedCore", tracked_core);
+            load_threshold = xfce_rc_read_int_entry (rc, "LoadThreshold", load_threshold);
 
-            if ((value = xfce_rc_read_entry (rc, "Foreground1", NULL)))
-                gdk_rgba_parse (&foreground1, value);
-            if ((value = xfce_rc_read_entry (rc, "Foreground2", NULL)))
-                gdk_rgba_parse (&foreground2, value);
-            if ((value = xfce_rc_read_entry (rc, "Foreground3", NULL)))
-                gdk_rgba_parse (&foreground3, value);
-            if ((value = xfce_rc_read_entry (rc, "Background", NULL)))
-                gdk_rgba_parse (&background, value);
-            if ((value = xfce_rc_read_entry (rc, "BarsColor", NULL))) {
-                gdk_rgba_parse (&barscolor, value);
-                base->has_barcolor = TRUE;
+            if ((value = xfce_rc_read_entry (rc, "Command", NULL))) {
+                command = g_strdup (value);
+            }
+
+            for (i = 0; i < NUM_COLORS; i++)
+            {
+                if ((value = xfce_rc_read_entry (rc, color_keys[i], NULL)))
+                {
+                    gdk_rgba_parse (&colors[i], value);
+                    if (i == BARS_COLOR)
+                        base->has_barcolor = TRUE;
+                }
+            }
+
+            switch (mode)
+            {
+                case MODE_DISABLED:
+                case MODE_NORMAL:
+                case MODE_LED:
+                case MODE_NO_HISTORY:
+                case MODE_GRID:
+                    break;
+                default:
+                    mode = MODE_NORMAL;
             }
 
             xfce_rc_close (rc);
@@ -140,17 +132,18 @@ read_settings (XfcePanelPlugin *plugin, CPUGraph *base)
     set_mode (base, mode);
     set_color_mode (base, color_mode);
     set_frame (base, frame);
-    set_command (base, associated_command);
+    if (command)
+        set_command (base, command);
     set_in_terminal (base, in_terminal);
     set_startup_notification (base, startup_notification);
     set_border (base, border);
     set_tracked_core (base, tracked_core);
     set_bars (base, bars);
-    set_color (base, 1, foreground1);
-    set_color (base, 2, foreground2);
-    set_color (base, 3, foreground3);
-    set_color (base, 0, background);
-    set_color (base, 4, barscolor);
+    set_smt (base, highlight_smt);
+    for (i = 0; i < NUM_COLORS; i++)
+        set_color (base, i, colors[i]);
+    set_load_threshold (base, load_threshold * 0.01f);
+    g_free (command);
 }
 
 void
@@ -158,6 +151,7 @@ write_settings (XfcePanelPlugin *plugin, CPUGraph *base)
 {
     XfceRc *rc;
     char *file;
+    guint i;
 
     if (!(file = xfce_panel_plugin_save_location (plugin, TRUE)))
         return;
@@ -176,16 +170,44 @@ write_settings (XfcePanelPlugin *plugin, CPUGraph *base)
     xfce_rc_write_int_entry (rc, "Border", base->has_border);
     xfce_rc_write_int_entry (rc, "Bars", base->has_bars);
     xfce_rc_write_int_entry (rc, "TrackedCore", base->tracked_core);
-    xfce_rc_write_entry (rc, "Command", base->command ? base->command : "");
+    if (base->command)
+        xfce_rc_write_entry (rc, "Command", base->command);
+    else
+        xfce_rc_delete_entry (rc, "Command", FALSE);
     xfce_rc_write_int_entry (rc, "InTerminal", base->in_terminal);
     xfce_rc_write_int_entry (rc, "StartupNotification", base->startup_notification);
     xfce_rc_write_int_entry (rc, "ColorMode", base->color_mode);
+    if (base->load_threshold != 0)
+        xfce_rc_write_int_entry (rc, "LoadThreshold", (gint) roundf (100 * base->load_threshold));
+    else
+        xfce_rc_delete_entry (rc, "LoadThreshold", FALSE);
+
+    for (i = 0; i < NUM_COLORS; i++)
+    {
+        const gchar *key = color_keys[i];
+
+        if(i == BARS_COLOR && !base->has_barcolor)
+            key = NULL;
+
+        if (key)
+        {
+            gchar *rgba = gdk_rgba_to_string (&base->colors[i]);
+            gchar *rgba_default = gdk_rgba_to_string (&default_colors[i]);
+
+            if (strcmp (rgba, rgba_default) != 0)
+                xfce_rc_write_entry (rc, key, rgba);
+            else
+                xfce_rc_delete_entry (rc, key, FALSE);
+
+            g_free (rgba);
+            g_free (rgba_default);
+        }
+    }
+
+    if (base->highlight_smt != HIGHLIGHT_SMT_BY_DEFAULT)
+        xfce_rc_write_int_entry (rc, "SmtIssues", base->highlight_smt);
+    else
+        xfce_rc_delete_entry (rc, "SmtIssues", FALSE);
 
-    xfce_rc_write_entry (rc, "Foreground1", gdk_rgba_to_string (&(base->colors[1])));
-    xfce_rc_write_entry (rc, "Foreground2", gdk_rgba_to_string (&(base->colors[2])));
-    xfce_rc_write_entry (rc, "Foreground3", gdk_rgba_to_string (&(base->colors[3])));
-    xfce_rc_write_entry (rc, "Background", gdk_rgba_to_string (&(base->colors[0])));
-    if (base->has_barcolor)
-        xfce_rc_write_entry (rc, "BarsColor", gdk_rgba_to_string (&(base->colors[4])));
     xfce_rc_close (rc);
 }
